@@ -16,7 +16,7 @@ use POE::Component::IRC::Plugin qw(:ALL);
 use base qw(POE::Component::IRC);
 use vars qw($VERSION);
 
-$VERSION = '1.5';
+$VERSION = '1.7';
 
 # Event handlers for tracking the STATE. $self->{STATE} is used as our namespace.
 # u_irc() is used to create unique keys.
@@ -64,13 +64,25 @@ sub S_join {
   my $channel = ${ $_[1] };
   my $uchan = u_irc $channel, $mapping;
   my $unick = u_irc $nick, $mapping;
+  my $excepts = $irc->isupport('EXCEPTS');
+  my $invex = $irc->isupport('INVEX');
 
   if ( $unick eq u_irc ( $self->nick_name(), $mapping ) ) {
 	delete $self->{STATE}->{Chans}->{ $uchan };
-	$self->{CHANNEL_SYNCH}->{ $uchan } = { MODE => 0, WHO => 0 };
-        $self->{STATE}->{Chans}->{ $uchan } = { };
+	$self->{CHANNEL_SYNCH}->{ $uchan } = { MODE => 0, WHO => 0, BAN => 0 };
+        $self->{STATE}->{Chans}->{ $uchan } = { Name => $channel, Mode => '' };
+        if ($excepts) {
+          $self->{CHANNEL_SYNCH}->{ $uchan }->{EXCEPTS} = 0;
+          $irc->yield ( 'mode' => $channel => $excepts );
+        }
+        if ($invex) {
+          $self->{CHANNEL_SYNCH}->{ $uchan }->{INVEX} = 0;
+          $irc->yield ( 'mode' => $channel => $invex );
+        }
         $self->yield ( 'who' => $channel );
         $self->yield ( 'mode' => $channel );
+        $self->yield ( 'mode' => $channel => 'b');
+
   } else {
         $self->yield ( 'who' => $nick );
         $self->{STATE}->{Nicks}->{ $unick }->{Nick} = $nick;
@@ -196,14 +208,23 @@ sub S_mode {
   pop @_;
   my @modes = map { ${ $_ } } @_[2 .. $#_];
 
+  # CHANMODES is [$list_mode, $always_arg, $arg_when_set, $no_arg]
+  # A $list_mode always has an argument
+  my $statmodes = join '', keys %{ $irc->isupport('PREFIX') };
+  my $chanmodes = $irc->isupport('CHANMODES');
+  my $alwaysarg = join '', $statmodes,  @{ $chanmodes }[0 .. 1];
+
   # Do nothing if it is UMODE
   if ( $uchan ne u_irc ( $self->{RealNick}, $mapping ) ) {
      my $parsed_mode = parse_mode_line( @modes );
      while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
         my $arg;
-        $arg = shift ( @{ $parsed_mode->{args} } ) if ( $mode =~ /^(\+[hovklbIeaqfL]|-[hovbIeaq])/ );
+        $arg = shift ( @{ $parsed_mode->{args} } ) if ( $mode =~ /^(.[$alwaysarg]|\+[$chanmodes->[2]])/ );
+
+        $self->_send_event( 'irc_chan_mode', $who, $channel, $mode, $arg );
+
         SWITCH: {
-          if ( $mode =~ /\+([ohvaq])/ ) {
+          if ( $mode =~ /\+([$statmodes])/ ) {
                 my $flag = $1;
 		$arg = u_irc $arg, $mapping;
                 unless ($self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan }and $self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan } =~ /$flag/) {
@@ -212,7 +233,7 @@ sub S_mode {
                 }
                 last SWITCH;
           }
-          if ( $mode =~ /-([ohvaq])/ ) {
+          if ( $mode =~ /-([$statmodes])/ ) {
                 my $flag = $1;
 		$arg = u_irc $arg, $mapping;
                 if ($self->{STATE}->{Nicks}->{ $arg }->{CHANS}->{ $uchan } =~ /$flag/) {
@@ -221,29 +242,31 @@ sub S_mode {
                 }
                 last SWITCH;
           }
-          if ( $mode =~ /[bIefL]/ ) {
+          if ( $mode =~ /\+([$chanmodes->[0]])/ ) {
+                my $flag = $1;
+                $self->{STATE}->{Chans}->{ $uchan }->{Lists}->{ $flag }->{ $arg } = { SetBy => $who, SetAt => time() };
                 last SWITCH;
           }
-          if ( $mode eq '+l' and defined ( $arg ) ) {
-                $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= 'l' unless ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /l/ );
-                $self->{STATE}->{Chans}->{ $uchan }->{ChanLimit} = $arg;
+          if ( $mode =~ /-([$chanmodes->[0]])/ ) {
+                my $flag = $1;
+                delete $self->{STATE}->{Chans}->{ $uchan }->{Lists}->{ $flag }->{ $arg };
                 last SWITCH;
           }
-          if ( $mode eq '+k' and defined ( $arg ) ) {
-                $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= 'k' unless ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /k/ );
-                $self->{STATE}->{Chans}->{ $uchan }->{ChanKey} = $arg;
+
+          # All unhandled modes with arguments
+          if ( $mode =~ /\+([^$chanmodes->[3]])/ ) {
+                my $flag = $1;
+                $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= $flag unless $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /$flag/;
+                $self->{STATE}->{Chans}->{ $uchan }->{ModeArgs}->{ $flag } = $arg;
                 last SWITCH;
           }
-          if ( $mode eq '-l' ) {
-                $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ s/l//;
-                delete $self->{STATE}->{Chans}->{ $uchan }->{ChanLimit};
-                last SWITCH;
-          }
-          if ( $mode eq '-k' ) {
-                $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ s/k//;
-                delete $self->{STATE}->{Chans}->{ $uchan }->{ChanKey};
+          if ( $mode =~ /-([^$chanmodes->[3]])/ ) {
+                my $flag = $1;
+                $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ s/$flag//;
+                delete $self->{STATE}->{Chans}->{ $uchan }->{ModeArgs}->{ $flag };
                 last SWITCH;
           }
+
           # Anything else doesn't have arguments so just adjust {Mode} as necessary.
           if ( $mode =~ /^\+(.)/ ) {
                 my $flag = $1;
@@ -269,12 +292,24 @@ sub S_mode {
   return PCI_EAT_NONE;
 }
 
+sub S_topic {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  my $who = ${ $_[0] };
+  my $channel = ${ $_[1] };
+  my $uchan = u_irc $channel, $mapping;
+  my $topic = ${ $_[2] };
+
+  $self->{STATE}->{Chans}->{ $uchan }->{Topic} = { Value => $topic, SetBy => $who, SetAt => time() };
+
+  return PCI_EAT_NONE;  
+}
+
 # RPL_WHOREPLY
 sub S_352 {
   my ($self,$irc) = splice @_, 0, 2;
   my $mapping = $irc->isupport('CASEMAPPING');
-  my ($first,$second) = split(/ :/,${ $_[1] } );
-  my ($channel,$user,$host,$server,$nick,$status) = split(/ /,$first);
+  my ($channel,$user,$host,$server,$nick,$status,$second) = @{ ${ $_[2] } };
   my $real = substr($second,index($second," ")+1);
   my $unick = u_irc $nick, $mapping;
   my $uchan = u_irc $channel, $mapping;
@@ -286,14 +321,15 @@ sub S_352 {
   $self->{STATE}->{Nicks}->{ $unick }->{Server} = $server;
   if ( $channel ne '*' ) {
     my $whatever = '';
-    if ( $status =~ /\@/ ) { $whatever = 'o'; }
-    if ( $status =~ /\+/ ) { $whatever = 'v'; }
-    if ( $status =~ /\%/ ) { $whatever = 'h'; }
-    if ( $status =~ /\&/ ) { $whatever = 'a'; }
-    if ( $status =~ /\~/ ) { $whatever = 'q'; }
-    $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } = $whatever;
+    my $existing = $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } || '';    
+    my $prefix = $irc->isupport('PREFIX');
+    foreach my $mode ( keys %{ $prefix } ) {
+      $whatever .= $mode if ( $status =~ /\Q$prefix->{$mode}/ and $existing !~ /\Q$prefix->{$mode}/ );
+    }
+    $existing .= $whatever unless $existing and $existing =~ /$whatever/;
+    $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } = $existing;
+    $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick } = $existing;
     $self->{STATE}->{Chans}->{ $uchan }->{Name} = $channel;
-    $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick } = $whatever;
   }
   if ( $status =~ /\*/ ) {
     $self->{STATE}->{Nicks}->{ $unick }->{IRCop} = 1;
@@ -305,13 +341,13 @@ sub S_352 {
 sub S_315 {
   my ($self,$irc) = splice @_, 0, 2;
   my $mapping = $irc->isupport('CASEMAPPING');
-  my $channel = ( split / :/, ${ $_[1] } )[0];
+  #my $channel = ( split / :/, ${ $_[1] } )[0];
+  my $channel = ${ $_[2] }->[0];
   my $uchan = u_irc $channel, $mapping;
 
   # If it begins with #, &, + or ! its a channel apparently. RFC2812.
   if ( $channel =~ /^[\x23\x2B\x21\x26]/ ) {
-    $self->_channel_sync_who($channel);
-    if ( $self->_channel_sync($channel) ) {
+    if ( $self->_channel_sync($channel, 'WHO') ) {
 	delete $self->{CHANNEL_SYNCH}->{ $uchan };
 	$self->_send_event( 'irc_chan_sync', $channel );
     }
@@ -322,42 +358,158 @@ sub S_315 {
   return PCI_EAT_NONE;
 }
 
+# RPL_BANLIST
+sub S_367 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
+  my $channel = shift @args;
+  my $uchan = u_irc $channel, $mapping;
+  my ($mask, $who, $when) = @args;
+
+  $self->{STATE}->{Chans}->{ $uchan }->{Lists}->{b}->{ $mask } = { SetBy => $who, SetAt => $when };
+  return PCI_EAT_NONE;
+}
+
+# RPL_ENDOFBANLIST
+sub S_368 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
+  my $channel = shift @args;
+  my $uchan = u_irc $channel, $mapping;
+
+  if ( $self->_channel_sync($channel, 'BAN') ) {
+        delete $self->{CHANNEL_SYNCH}->{ $uchan };
+        $self->_send_event( 'irc_chan_sync', $channel );
+  }
+  return PCI_EAT_NONE;
+}
+
+# RPL_INVITELIST
+sub S_346 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
+  my $channel = shift @args;
+  my $uchan = u_irc $channel, $mapping;
+  my ($mask, $who, $when) = @args;
+  my $invex = $irc->isupport('INVEX');
+
+  $self->{STATE}->{Chans}->{ $uchan }->{Lists}->{ $invex }->{ $mask } = { SetBy => $who, SetAt => $when };
+  return PCI_EAT_NONE;
+}
+
+# RPL_ENDOFINVITELIST
+sub S_347 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
+  my $channel = shift @args;
+  my $uchan = u_irc $channel, $mapping;
+
+  if ( $self->_channel_sync($channel, 'INVEX') ) {
+        delete $self->{CHANNEL_SYNCH}->{ $uchan };
+        $self->_send_event( 'irc_chan_sync', $channel );
+  }
+  return PCI_EAT_NONE;
+}
+
+# RPL_EXCEPTLIST
+sub S_348 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
+  my $channel = shift @args;
+  my $uchan = u_irc $channel, $mapping;
+  my ($mask, $who, $when) = @args;
+  my $excepts = $irc->isupport('EXCEPTS');
+
+  $self->{STATE}->{Chans}->{ $uchan }->{Lists}->{ $excepts }->{ $mask } = { SetBy => $who, SetAt => $when };
+  return PCI_EAT_NONE;
+}
+
+# RPL_ENDOFEXCEPTLIST
+sub S_349 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
+  my $channel = shift @args;
+  my $uchan = u_irc $channel, $mapping;
+
+  if ( $self->_channel_sync($channel, 'EXCEPTS') ) {
+        delete $self->{CHANNEL_SYNCH}->{ $uchan };
+        $self->_send_event( 'irc_chan_sync', $channel );
+  }
+  return PCI_EAT_NONE;
+}
+
 # RPL_CHANNELMODEIS
 sub S_324 {
   my ($self,$irc) = splice @_, 0, 2;
   my $mapping = $irc->isupport('CASEMAPPING');
-  my @args = split / /, ${ $_[1] };
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
   my $channel = shift @args;
   my $uchan = u_irc $channel, $mapping;
+  my $chanmodes = $irc->isupport('CHANMODES');
 
   my $parsed_mode = parse_mode_line( @args );
   while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
         $mode =~ s/\+//;
         my $arg;
-        $arg = shift @{ $parsed_mode->{args} } if $mode =~ /[kl]/;
+        $arg = shift @{ $parsed_mode->{args} } if $mode =~ /[^$chanmodes->[3]]/; # doesn't match a mode with no args
 	if ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} ) {
           $self->{STATE}->{Chans}->{ $uchan }->{Mode} .= $mode unless $self->{STATE}->{Chans}->{ $uchan }->{Mode} =~ /$mode/;
 	} else {
 	  $self->{STATE}->{Chans}->{ $uchan }->{Mode} = $mode;
 	}
-        if ( $mode eq 'l' and defined ( $arg ) ) {
-           $self->{STATE}->{Chans}->{ $uchan }->{ChanLimit} = $arg;
-        }
-        if ( $mode eq 'k' and defined ( $arg ) ) {
-           $self->{STATE}->{Chans}->{ $uchan }->{ChanKey} = $arg;
-        }
+        $self->{STATE}->{Chans}->{ $uchan }->{ModeArgs}->{ $mode } = $arg if defined ( $arg );
   }
   if ( $self->{STATE}->{Chans}->{ $uchan }->{Mode} ) {
         $self->{STATE}->{Chans}->{ $uchan }->{Mode} = join('', sort {uc $a cmp uc $b} split //, $self->{STATE}->{Chans}->{ $uchan }->{Mode} );
   }
-  $self->_channel_sync_mode($channel);
-  if ( $self->_channel_sync($channel) ) {
+  if ( $self->_channel_sync($channel, 'MODE') ) {
 	delete $self->{CHANNEL_SYNCH}->{ $uchan };
 	$self->_send_event( 'irc_chan_sync', $channel );
   }
   return PCI_EAT_NONE;
 }
 
+# RPL_TOPIC
+sub S_332 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  my $channel = ${ $_[2] }->[0];
+  my $topic = ${ $_[2] }->[1];
+  my $uchan = u_irc $channel, $mapping;
+
+  $self->{STATE}->{Chans}->{ $uchan }->{Topic}->{Value} = $topic;
+
+  return PCI_EAT_NONE;
+}
+
+# RPL_TOPICWHOTIME
+sub S_333 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mapping = $irc->isupport('CASEMAPPING');
+  #my @args = split / /, ${ $_[1] };
+  my @args = @{ ${ $_[2] } };
+  my ($channel, $who, $when) = @args;
+  my $uchan = u_irc $channel, $mapping;
+
+  $self->{STATE}->{Chans}->{ $uchan }->{Topic}->{SetBy} = $who;
+  $self->{STATE}->{Chans}->{ $uchan }->{Topic}->{SetAt} = $when;
+
+  return PCI_EAT_NONE;
+}
+
 # Methods for STATE query
 # Internal methods begin with '_'
 #
@@ -366,43 +518,16 @@ sub _channel_sync {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
   my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $sync = $_[1];
 
-  return 0 unless $self->_channel_exists($channel);
+  return 0 unless ( $self->_channel_exists($channel) and defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) );
 
-  if ( defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) ) {
-	if ( $self->{CHANNEL_SYNCH}->{ $channel }->{MODE} and $self->{CHANNEL_SYNCH}->{ $channel }->{WHO} ) {
-		return 1;
-	}
+  if ($sync) { $self->{CHANNEL_SYNCH}->{ $channel }->{ $sync } = 1 }
+
+  for my $is_sync ( values %{ $self->{CHANNEL_SYNCH}->{ $channel } } ) {
+      return 0 unless $is_sync;
   }
-  return 0;
-}
-
-sub _channel_sync_mode {
-  my $self = shift;
-  my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-
-  return 0 unless $self->_channel_exists($channel);
-
-  if ( defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) ) {
-	$self->{CHANNEL_SYNCH}->{ $channel }->{MODE} = 1;
-	return 1;
-  }
-  return 0;
-}
-
-sub _channel_sync_who {
-  my $self = shift;
-  my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-
-  return 0 unless $self->_channel_exists($channel);
-
-  if ( defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) ) {
-	$self->{CHANNEL_SYNCH}->{ $channel }->{WHO} = 1;
-	return 1;
-  }
-  return 0;
+  return 1;
 }
 
 sub _nick_exists {
@@ -552,8 +677,8 @@ sub channel_limit {
 
   return unless $self->_channel_exists($channel);
 
-  if ( $self->is_channel_mode_set($channel,'l') and defined ( $self->{STATE}->{Chans}->{ $channel }->{ChanLimit} ) ) {
-	return $self->{STATE}->{Chans}->{ $channel }->{ChanLimit};
+  if ( $self->is_channel_mode_set($channel,'l') and defined ( $self->{STATE}->{Chans}->{ $channel }->{ModeArgs}->{l} ) ) {
+	return $self->{STATE}->{Chans}->{ $channel }->{ModeArgs}->{l};
   }
   return undef;
 }
@@ -565,8 +690,8 @@ sub channel_key {
 
   return unless $self->_channel_exists($channel);
 
-  if ( $self->is_channel_mode_set($channel,'k') and defined ( $self->{STATE}->{Chans}->{ $channel }->{ChanKey} ) ) {
-	return $self->{STATE}->{Chans}->{ $channel }->{ChanKey};
+  if ( $self->is_channel_mode_set($channel,'k') and defined ( $self->{STATE}->{Chans}->{ $channel }->{ModeArgs}->{k} ) ) {
+	return $self->{STATE}->{Chans}->{ $channel }->{ModeArgs}->{k};
   }
   return undef;
 }
@@ -663,6 +788,68 @@ sub ban_mask {
   return @result;
 }
 
+sub channel_ban_list {
+  my $self = shift;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my %result;
+
+  return undef unless $self->_channel_exists($channel);
+
+  if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Lists}->{b} ) ) {
+    %result = %{ $self->{STATE}->{Chans}->{ $channel }->{Lists}->{b} };
+  }
+
+  return \%result;
+}
+
+sub channel_except_list {
+  my $self = shift;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my $excepts = $self->isupport('EXCEPTS');
+  my %result;
+
+  return undef unless $self->_channel_exists($channel);
+
+  if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Lists}->{ $excepts } ) ) {
+    %result = %{ $self->{STATE}->{Chans}->{ $channel }->{Lists}->{ $excepts } };
+  }
+
+  return \%result;
+}
+
+sub channel_invex_list {
+  my $self = shift;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my $invex = $self->isupport('INVEX');
+  my %result;
+
+  return undef unless $self->_channel_exists($channel);
+
+  if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Lists}->{ $invex } ) ) {
+    %result = %{ $self->{STATE}->{Chans}->{ $channel }->{Lists}->{ $invex } };
+  }
+
+  return \%result;
+}
+
+sub channel_topic {
+  my $self = shift;
+  my $mapping = $self->isupport('CASEMAPPING');
+  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my %result;
+
+  return undef unless $self->_channel_exists($channel);
+
+  if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Topic} ) ) {
+    %result = %{ $self->{STATE}->{Chans}->{ $channel }->{Topic} };
+  }
+
+  return \%result;
+}
+
 1;
 
 =head1 NAME
@@ -772,7 +959,7 @@ to query the collected state.
 
 =head1 METHODS
 
-All of the L<POE::Component::IRC|POE::Component::IRC> methods are supported, plus the following:
+All of the L<POE::Component::IRC> methods are supported, plus the following:
 
 =over
 
@@ -823,6 +1010,16 @@ Expects a channel as parameter. Returns the channel key or undef.
 Expects a channel and a nickname as parameters. Returns 1 if the specified nick is on the specified channel or 0
 otherwise. If either channel or nick does not exist in the state then a 0 will be returned.
 
+=item is_channel_owner
+
+Expects a channel and a nickname as parameters. Returns 1 if the specified nick is an owner on the specified channel or 0
+otherwise. If either channel or nick does not exist in the state then a 0 will be returned.
+
+=item is_channel_admin
+
+Expects a channel and a nickname as parameters. Returns 1 if the specified nick is an admin on the specified channel or 0
+otherwise. If either channel or nick does not exist in the state then a 0 will be returned.
+
 =item is_channel_operator
 
 Expects a channel and a nickname as parameters. Returns 1 if the specified nick is an operator on the specified channel or 0
@@ -850,18 +1047,42 @@ will be returned if the nickname does not exist in the state.
 =item nick_info
 
 Expects a nickname. Returns a hashref containing similar information to that returned by WHOIS. Returns an undef
-if the nickname doesn't exist in the state. The hashref contains the following keys: 'Nick', 'User', 'Host', 'Userhost', 'Server' and, if applicable, 'IRCop'.
+if the nickname doesn't exist in the state. The hashref contains the following keys: 'Nick', 'User', 'Host', 'Userhost', 'Real', 'Server' and, if applicable, 'IRCop'.
 
 =item ban_mask
 
 Expects a channel and a ban mask, as passed to MODE +b-b. Returns a list of nicks on that channel that match the specified
 ban mask or an empty list if the channel doesn't exist in the state or there are no matches.
 
+=item channel_ban_list
+
+Expects a channel as a parameter. Returns a hashref containing the banlist if the channel is in the state, undef if not.
+The hashref keys are the entries on the list, each with the keys 'SetBy' and 'SetAt'. These keys will hold the nick!hostmask of
+the user who set the entry (or just the nick if it's all the ircd gives us), and the time at which it was set respectively.
+
+=item channel_invex_list
+
+Expects a channel as a parameter. Returns a hashref containing the invite exception list if the channel is in the state, undef if not.
+The hashref keys are the entries on the list, each with the keys 'SetBy' and 'SetAt'. These keys will hold the nick!hostmask of
+the user who set the entry (or just the nick if it's all the ircd gives us), and the time at which it was set respectively.
+
+=item channel_except_list                                             
+
+Expects a channel as a parameter. Returns a hashref containing the ban exception list if the channel is in the state, undef if not.
+The hashref keys are the entries on the list, each with the keys 'SetBy' and 'SetAt'. These keys will hold the nick!hostmask of
+the user who set the entry (or just the nick if it's all the ircd gives us), and the time at which it was set respectively.
+
+=item channel_topic
+
+Expects a channel as a parameter. Returns a hashref containing topic information if the channel is in the state, undef if not.
+The hashref contains the following keys: 'Value', 'SetBy', 'SetAt'. These keys will hold the topic itself, the nick!hostmask of
+the user who set it (or just the nick if it's all the ircd gives us), and the time at which it was set respectively.
+
 =back
 
 =head1 OUTPUT
 
-As well as all the usual L<POE::Component::IRC|POE::Component::IRC> 'irc_*' events, there are the following events you can register for:
+As well as all the usual L<POE::Component::IRC> 'irc_*' events, there are the following events you can register for:
 
 =over
 
@@ -874,9 +1095,14 @@ Sent whenever the component has completed synchronising a channel that it has jo
 Sent whenever the component has completed synchronising a user who has joined a channel the component is on.
 ARG0 is the user's nickname.
 
+=item irc_chan_mode
+
+This is almost identical to irc_mode, except that it's sent once for each individual mode with it's respective
+argument if it has one (ie. the banmask if it's +b or -b). However, this event is only sent for channel modes.
+
 =back
 
-The following two 'irc_*' events are the same as their L<POE::Component::IRC|POE::Component::IRC> counterparts,
+The following two 'irc_*' events are the same as their L<POE::Component::IRC> counterparts,
 with the additional parameters:
 
 =over
@@ -901,6 +1127,8 @@ You may want to ignore these. When someones joins a channel the bot is on, it is
 
 Chris Williams <chris@bingosnet.co.uk>
 
+With contributions from the Kinky Black Goat.
+
 =head1 SEE ALSO
 
-L<POE::Component::IRC|POE::Component::IRC>
+L<POE::Component::IRC>
